@@ -1,6 +1,8 @@
-package swarm // import "github.com/docker/docker/api/types/swarm"
+package swarm
 
-import "os"
+import (
+	"os"
+)
 
 // Config represents a config.
 type Config struct {
@@ -12,6 +14,12 @@ type Config struct {
 // ConfigSpec represents a config specification from a config in swarm
 type ConfigSpec struct {
 	Annotations
+
+	// Data is the data to store as a config.
+	//
+	// The maximum allowed size is 1000KB, as defined in [MaxConfigSize].
+	//
+	// [MaxConfigSize]: https://pkg.go.dev/github.com/moby/swarmkit/v2@v2.0.0-20250103191802-8c1959736554/manager/controlapi#MaxConfigSize
 	Data []byte `json:",omitempty"`
 
 	// Templating controls whether and how to evaluate the config payload as
@@ -38,3 +46,10 @@ type ConfigReference struct {
 	ConfigID   string
 	ConfigName string
 }
+
+// ConfigCreateResponse contains the information returned to a client
+// on the creation of a new config.
+type ConfigCreateResponse struct {
+	// ID is the id of the created config.
+	ID string
+}
