@@ -1,36 +1,31 @@
 package image
 
 import (
-	"context"
-
 	"github.com/docker/cli/cli"
 	"github.com/docker/cli/cli/command"
 	"github.com/docker/cli/cli/command/completion"
+	"github.com/moby/moby/client"
 	"github.com/spf13/cobra"
 )
 
-type tagOptions struct {
-	image string
-	name  string
-}
-
-// NewTagCommand creates a new `docker tag` command
-func NewTagCommand(dockerCli command.Cli) *cobra.Command {
-	var opts tagOptions
-
+// newTagCommand creates a new "docker image tag" command.
+func newTagCommand(dockerCLI command.Cli) *cobra.Command {
 	cmd := &cobra.Command{
 		Use:   "tag SOURCE_IMAGE[:TAG] TARGET_IMAGE[:TAG]",
 		Short: "Create a tag TARGET_IMAGE that refers to SOURCE_IMAGE",
 		Args:  cli.ExactArgs(2),
 		RunE: func(cmd *cobra.Command, args []string) error {
-			opts.image = args[0]
-			opts.name = args[1]
-			return runTag(cmd.Context(), dockerCli, opts)
+			_, err := dockerCLI.Client().ImageTag(cmd.Context(), client.ImageTagOptions{
+				Source: args[0],
+				Target: args[1],
+			})
+			return err
 		},
 		Annotations: map[string]string{
 			"aliases": "docker image tag, docker tag",
 		},
-		ValidArgsFunction: completion.ImageNames(dockerCli),
+		ValidArgsFunction:     completion.ImageNames(dockerCLI, 2),
+		DisableFlagsInUseLine: true,
 	}
 
 	flags := cmd.Flags()
@@ -38,7 +33,3 @@ func NewTagCommand(dockerCli command.Cli) *cobra.Command {
 
 	return cmd
 }
-
-func runTag(ctx context.Context, dockerCli command.Cli, opts tagOptions) error {
-	return dockerCli.Client().ImageTag(ctx, opts.image, opts.name)
-}
