@@ -1,4 +1,4 @@
-package daemon // import "github.com/docker/docker/daemon"
+package daemon
 
 import (
 	"context"
@@ -6,13 +6,15 @@ import (
 	"io"
 
 	"github.com/containerd/log"
-	"github.com/docker/docker/api/types/backend"
-	"github.com/docker/docker/api/types/events"
-	"github.com/docker/docker/container"
-	"github.com/docker/docker/container/stream"
-	"github.com/docker/docker/daemon/logger"
-	"github.com/docker/docker/errdefs"
-	"github.com/docker/docker/pkg/stdcopy"
+	"github.com/moby/moby/api/pkg/stdcopy"
+	containertypes "github.com/moby/moby/api/types/container"
+	"github.com/moby/moby/api/types/events"
+	"github.com/moby/moby/v2/daemon/container"
+	"github.com/moby/moby/v2/daemon/internal/stdcopymux"
+	"github.com/moby/moby/v2/daemon/internal/stream"
+	"github.com/moby/moby/v2/daemon/logger"
+	"github.com/moby/moby/v2/daemon/server/backend"
+	"github.com/moby/moby/v2/errdefs"
 	"github.com/moby/term"
 	"github.com/pkg/errors"
 )
@@ -32,13 +34,11 @@ func (daemon *Daemon) ContainerAttach(prefixOrName string, req *backend.Containe
 	if err != nil {
 		return err
 	}
-	if ctr.IsPaused() {
-		err := fmt.Errorf("container %s is paused, unpause the container before attach", prefixOrName)
-		return errdefs.Conflict(err)
+	if ctr.State.IsPaused() {
+		return errdefs.Conflict(fmt.Errorf("container %s is paused, unpause the container before attach", prefixOrName))
 	}
-	if ctr.IsRestarting() {
-		err := fmt.Errorf("container %s is restarting, wait until the container is running", prefixOrName)
-		return errdefs.Conflict(err)
+	if ctr.State.IsRestarting() {
+		return errdefs.Conflict(fmt.Errorf("container %s is restarting, wait until the container is running", prefixOrName))
 	}
 
 	cfg := stream.AttachConfig{
@@ -51,8 +51,6 @@ func (daemon *Daemon) ContainerAttach(prefixOrName string, req *backend.Containe
 	}
 	ctr.StreamConfig.AttachStreams(&cfg)
 
-	multiplexed := !ctr.Config.Tty && req.MuxStreams
-
 	clientCtx, closeNotify := context.WithCancel(context.Background())
 	defer closeNotify()
 	go func() {
@@ -68,6 +66,7 @@ func (daemon *Daemon) ContainerAttach(prefixOrName string, req *backend.Containe
 		}
 	}()
 
+	multiplexed := !ctr.Config.Tty && req.MuxStreams
 	inStream, outStream, errStream, err := req.GetStreams(multiplexed, closeNotify)
 	if err != nil {
 		return err
@@ -76,8 +75,8 @@ func (daemon *Daemon) ContainerAttach(prefixOrName string, req *backend.Containe
 	defer inStream.Close()
 
 	if multiplexed {
-		errStream = stdcopy.NewStdWriter(errStream, stdcopy.Stderr)
-		outStream = stdcopy.NewStdWriter(outStream, stdcopy.Stdout)
+		errStream = stdcopymux.NewStdWriter(errStream, stdcopy.Stderr)
+		outStream = stdcopymux.NewStdWriter(outStream, stdcopy.Stdout)
 	}
 
 	if cfg.UseStdin {
@@ -91,7 +90,7 @@ func (daemon *Daemon) ContainerAttach(prefixOrName string, req *backend.Containe
 	}
 
 	if err := daemon.containerAttach(ctr, &cfg, req.Logs, req.Stream); err != nil {
-		fmt.Fprintf(outStream, "Error attaching: %s\n", err)
+		_, _ = fmt.Fprintln(outStream, "Error attaching:", err)
 	}
 	return nil
 }
@@ -124,16 +123,19 @@ func (daemon *Daemon) ContainerAttachRaw(prefixOrName string, stdin io.ReadClose
 	return daemon.containerAttach(ctr, &cfg, false, doStream)
 }
 
-func (daemon *Daemon) containerAttach(c *container.Container, cfg *stream.AttachConfig, logs, doStream bool) error {
-	if logs {
-		logDriver, logCreated, err := daemon.getLogger(c)
+func (daemon *Daemon) containerAttach(ctr *container.Container, cfg *stream.AttachConfig, enableLogs, doStream bool) error {
+	if enableLogs {
+		logDriver, logCreated, err := daemon.getLogger(ctr)
 		if err != nil {
 			return err
 		}
 		if logCreated {
 			defer func() {
 				if err = logDriver.Close(); err != nil {
-					log.G(context.TODO()).Errorf("Error closing logger: %v", err)
+					log.G(context.TODO()).WithFields(log.Fields{
+						"error":     err,
+						"container": ctr.ID,
+					}).Error("Error closing logger")
 				}
 			}()
 		}
@@ -141,13 +143,13 @@ func (daemon *Daemon) containerAttach(c *container.Container, cfg *stream.Attach
 		if !ok {
 			return logger.ErrReadLogsNotSupported{}
 		}
-		logs := cLog.ReadLogs(context.TODO(), logger.ReadConfig{Tail: -1})
-		defer logs.ConsumerGone()
+		logWatcher := cLog.ReadLogs(context.TODO(), logger.ReadConfig{Tail: -1})
+		defer logWatcher.ConsumerGone()
 
 	LogLoop:
 		for {
 			select {
-			case msg, ok := <-logs.Msg:
+			case msg, ok := <-logWatcher.Msg:
 				if !ok {
 					break LogLoop
 				}
@@ -157,14 +159,17 @@ func (daemon *Daemon) containerAttach(c *container.Container, cfg *stream.Attach
 				if msg.Source == "stderr" && cfg.Stderr != nil {
 					cfg.Stderr.Write(msg.Line)
 				}
-			case err := <-logs.Err:
-				log.G(context.TODO()).Errorf("Error streaming logs: %v", err)
+			case err := <-logWatcher.Err:
+				log.G(context.TODO()).WithFields(log.Fields{
+					"error":     err,
+					"container": ctr.ID,
+				}).Error("Error streaming logs")
 				break LogLoop
 			}
 		}
 	}
 
-	daemon.LogContainerEvent(c, events.ActionAttach)
+	daemon.LogContainerEvent(ctr, events.ActionAttach)
 
 	if !doStream {
 		return nil
@@ -173,33 +178,38 @@ func (daemon *Daemon) containerAttach(c *container.Container, cfg *stream.Attach
 	if cfg.Stdin != nil {
 		r, w := io.Pipe()
 		go func(stdin io.ReadCloser) {
-			defer w.Close()
-			defer log.G(context.TODO()).Debug("Closing buffered stdin pipe")
 			io.Copy(w, stdin)
+			log.G(context.TODO()).WithFields(log.Fields{
+				"container": ctr.ID,
+			}).Debug("Closing buffered stdin pipe")
+			w.Close()
 		}(cfg.Stdin)
 		cfg.Stdin = r
 	}
 
-	if !c.Config.OpenStdin {
+	if !ctr.Config.OpenStdin {
 		cfg.Stdin = nil
 	}
 
-	if c.Config.StdinOnce && !c.Config.Tty {
+	if ctr.Config.StdinOnce && !ctr.Config.Tty {
 		// Wait for the container to stop before returning.
-		waitChan := c.Wait(context.Background(), container.WaitConditionNotRunning)
+		waitChan := ctr.State.Wait(context.Background(), containertypes.WaitConditionNotRunning)
 		defer func() {
 			<-waitChan // Ignore returned exit code.
 		}()
 	}
 
-	ctx := c.AttachContext()
-	err := <-c.StreamConfig.CopyStreams(ctx, cfg)
+	ctx := ctr.AttachContext()
+	err := <-ctr.StreamConfig.CopyStreams(ctx, cfg)
 	if err != nil {
 		var ierr term.EscapeError
 		if errors.Is(err, context.Canceled) || errors.As(err, &ierr) {
-			daemon.LogContainerEvent(c, events.ActionDetach)
+			daemon.LogContainerEvent(ctr, events.ActionDetach)
 		} else {
-			log.G(ctx).Errorf("attach failed with error: %v", err)
+			log.G(ctx).WithFields(log.Fields{
+				"error":     err,
+				"container": ctr.ID,
+			}).Error("attach failed with error")
 		}
 	}
 
