@@ -1,6 +1,7 @@
 //go:build windows
 
-package container // import "github.com/docker/docker/daemon/cluster/executor/container"
+package container
+
 import (
 	"strings"
 	"testing"
