@@ -1,7 +1,7 @@
-package daemon // import "github.com/docker/docker/daemon"
+package daemon
 
 import (
-	swarmtypes "github.com/docker/docker/api/types/swarm"
+	swarmtypes "github.com/moby/moby/api/types/swarm"
 )
 
 // SetContainerConfigReferences sets the container config references needed
