@@ -1,38 +1,34 @@
-// FIXME(thaJeztah): remove once we are a module; the go:build directive prevents go from downgrading language version to go1.16:
-//go:build go1.23
-
-package daemon // import "github.com/docker/docker/daemon"
+package daemon
 
 import (
 	"context"
 	"errors"
 	"fmt"
-	"net"
+	"net/netip"
 	"os"
 	"runtime"
+	"slices"
 	"strings"
 	"time"
 
 	cerrdefs "github.com/containerd/errdefs"
 	"github.com/containerd/log"
-	containertypes "github.com/docker/docker/api/types/container"
-	"github.com/docker/docker/api/types/events"
-	networktypes "github.com/docker/docker/api/types/network"
-	"github.com/docker/docker/container"
-	"github.com/docker/docker/daemon/config"
-	"github.com/docker/docker/daemon/network"
-	"github.com/docker/docker/errdefs"
-	"github.com/docker/docker/internal/metrics"
-	"github.com/docker/docker/internal/multierror"
-	"github.com/docker/docker/internal/sliceutil"
-	"github.com/docker/docker/libnetwork"
-	"github.com/docker/docker/libnetwork/netlabel"
-	"github.com/docker/docker/libnetwork/scope"
-	"github.com/docker/docker/libnetwork/types"
-	"github.com/docker/docker/opts"
-	"github.com/docker/docker/pkg/stringid"
-	"github.com/docker/docker/runconfig"
-	"github.com/docker/go-connections/nat"
+	containertypes "github.com/moby/moby/api/types/container"
+	"github.com/moby/moby/api/types/events"
+	networktypes "github.com/moby/moby/api/types/network"
+	"github.com/moby/moby/v2/daemon/config"
+	"github.com/moby/moby/v2/daemon/container"
+	"github.com/moby/moby/v2/daemon/internal/metrics"
+	"github.com/moby/moby/v2/daemon/internal/multierror"
+	"github.com/moby/moby/v2/daemon/internal/stringid"
+	"github.com/moby/moby/v2/daemon/libnetwork"
+	"github.com/moby/moby/v2/daemon/libnetwork/netlabel"
+	"github.com/moby/moby/v2/daemon/libnetwork/scope"
+	"github.com/moby/moby/v2/daemon/libnetwork/types"
+	"github.com/moby/moby/v2/daemon/network"
+	"github.com/moby/moby/v2/daemon/pkg/opts"
+	"github.com/moby/moby/v2/errdefs"
+	"github.com/moby/moby/v2/internal/sliceutil"
 	"go.opentelemetry.io/otel"
 	"go.opentelemetry.io/otel/attribute"
 	"go.opentelemetry.io/otel/trace"
@@ -40,14 +36,6 @@ import (
 
 const errSetupNetworking = "failed to set up container networking"
 
-func ipAddresses(ips []net.IP) []string {
-	var addrs []string
-	for _, ip := range ips {
-		addrs = append(addrs, ip.String())
-	}
-	return addrs
-}
-
 func buildSandboxOptions(cfg *config.Config, ctr *container.Container) ([]libnetwork.SandboxOption, error) {
 	var sboxOptions []libnetwork.SandboxOption
 	sboxOptions = append(sboxOptions, libnetwork.OptionHostname(ctr.Config.Hostname), libnetwork.OptionDomainname(ctr.Config.Domainname))
@@ -60,15 +48,18 @@ func buildSandboxOptions(cfg *config.Config, ctr *container.Container) ([]libnet
 		sboxOptions = append(sboxOptions, libnetwork.OptionUseExternalKey())
 	}
 
-	// Add platform-specific Sandbox options.
-	if err := buildSandboxPlatformOptions(ctr, cfg, &sboxOptions); err != nil {
+	// Update the container with platform-specific options, and
+	// add platform-specific Sandbox options.
+	platformOpts, err := buildSandboxPlatformOptions(ctr, cfg)
+	if err != nil {
 		return nil, err
 	}
+	sboxOptions = append(sboxOptions, platformOpts...)
 
 	if len(ctr.HostConfig.DNS) > 0 {
 		sboxOptions = append(sboxOptions, libnetwork.OptionDNS(ctr.HostConfig.DNS))
 	} else if len(cfg.DNS) > 0 {
-		sboxOptions = append(sboxOptions, libnetwork.OptionDNS(ipAddresses(cfg.DNS)))
+		sboxOptions = append(sboxOptions, libnetwork.OptionDNS(cfg.DNS))
 	}
 	if len(ctr.HostConfig.DNSSearch) > 0 {
 		sboxOptions = append(sboxOptions, libnetwork.OptionDNSSearch(ctr.HostConfig.DNSSearch))
@@ -92,70 +83,66 @@ func buildSandboxOptions(cfg *config.Config, ctr *container.Container) ([]libnet
 		// config variable
 		if ip == opts.HostGatewayName {
 			if len(cfg.HostGatewayIPs) == 0 {
-				return nil, fmt.Errorf("unable to derive the IP value for host-gateway")
+				return nil, errors.New("unable to derive the IP value for host-gateway")
 			}
 			for _, gip := range cfg.HostGatewayIPs {
-				sboxOptions = append(sboxOptions, libnetwork.OptionExtraHost(host, gip.String()))
+				sboxOptions = append(sboxOptions, libnetwork.OptionExtraHost(host, gip.Unmap()))
 			}
 		} else {
-			sboxOptions = append(sboxOptions, libnetwork.OptionExtraHost(host, ip))
+			sboxOptions = append(sboxOptions, libnetwork.OptionExtraHost(host, netip.MustParseAddr(ip).Unmap()))
 		}
 	}
 
-	bindings := make(nat.PortMap)
-	if ctr.HostConfig.PortBindings != nil {
-		for p, b := range ctr.HostConfig.PortBindings {
-			bindings[p] = []nat.PortBinding{}
-			for _, bb := range b {
-				bindings[p] = append(bindings[p], nat.PortBinding{
-					HostIP:   bb.HostIP,
-					HostPort: bb.HostPort,
-				})
-			}
-		}
+	portBindings := make(networktypes.PortMap, len(ctr.HostConfig.PortBindings))
+	for p, b := range ctr.HostConfig.PortBindings {
+		portBindings[p] = slices.Clone(b)
 	}
 
-	// TODO(thaJeztah): Move this code to a method on nat.PortSet.
-	ports := make([]nat.Port, 0, len(ctr.Config.ExposedPorts))
 	for p := range ctr.Config.ExposedPorts {
-		ports = append(ports, p)
+		if _, ok := portBindings[p]; !ok {
+			// Create nil entries for exposed but un-mapped ports.
+			portBindings[p] = nil
+		}
 	}
-	nat.SortPortMap(ports, bindings)
 
 	var (
 		publishedPorts []types.PortBinding
 		exposedPorts   []types.TransportPort
 	)
-	for _, port := range ports {
-		portProto := types.ParseProtocol(port.Proto())
-		portNum := uint16(port.Int())
+	for port, bindings := range portBindings {
+		protocol := types.ParseProtocol(string(port.Proto()))
 		exposedPorts = append(exposedPorts, types.TransportPort{
-			Proto: portProto,
-			Port:  portNum,
+			Proto: protocol,
+			Port:  port.Num(),
 		})
 
-		for _, binding := range bindings[port] {
-			newP, err := nat.NewPort(nat.SplitProtoPort(binding.HostPort))
-			var portStart, portEnd int
-			if err == nil {
-				portStart, portEnd, err = newP.Range()
-			}
-			if err != nil {
-				return nil, fmt.Errorf("Error parsing HostPort value(%s):%v", binding.HostPort, err)
+		for _, binding := range bindings {
+			var (
+				portRange networktypes.PortRange
+				err       error
+			)
+
+			// Empty HostPort means to map to an ephemeral port.
+			if binding.HostPort != "" {
+				portRange, err = networktypes.ParsePortRange(binding.HostPort)
+				if err != nil {
+					return nil, fmt.Errorf("error parsing HostPort value(%s):%v", binding.HostPort, err)
+				}
 			}
+
 			publishedPorts = append(publishedPorts, types.PortBinding{
-				Proto:       portProto,
-				Port:        portNum,
-				HostIP:      net.ParseIP(binding.HostIP),
-				HostPort:    uint16(portStart),
-				HostPortEnd: uint16(portEnd),
+				Proto:       protocol,
+				Port:        port.Num(),
+				HostIP:      binding.HostIP.AsSlice(),
+				HostPort:    portRange.Start(),
+				HostPortEnd: portRange.End(),
 			})
 		}
 
-		if ctr.HostConfig.PublishAllPorts && len(bindings[port]) == 0 {
+		if ctr.HostConfig.PublishAllPorts && len(bindings) == 0 {
 			publishedPorts = append(publishedPorts, types.PortBinding{
-				Proto: portProto,
-				Port:  portNum,
+				Proto: protocol,
+				Port:  port.Num(),
 			})
 		}
 	}
@@ -174,7 +161,7 @@ func (daemon *Daemon) updateNetworkSettings(ctr *container.Container, n *libnetw
 	}
 
 	if !ctr.HostConfig.NetworkMode.IsHost() && containertypes.NetworkMode(n.Type()).IsHost() {
-		return runconfig.ErrConflictConnectToHostNetwork
+		return cerrdefs.ErrInvalidArgument.WithMessage("cannot connect container to host network - container must be created in host network mode")
 	}
 
 	for s, v := range ctr.NetworkSettings.Networks {
@@ -194,13 +181,11 @@ func (daemon *Daemon) updateNetworkSettings(ctr *container.Container, n *libnetw
 			// Avoid duplicate config
 			return nil
 		}
-		if !containertypes.NetworkMode(sn.Type()).IsPrivate() ||
-			!containertypes.NetworkMode(n.Type()).IsPrivate() {
-			return runconfig.ErrConflictSharedNetwork
+		if !containertypes.NetworkMode(sn.Type()).IsPrivate() || !containertypes.NetworkMode(n.Type()).IsPrivate() {
+			return cerrdefs.ErrInvalidArgument.WithMessage("container sharing network namespace with another container or host cannot be connected to any other network")
 		}
-		if containertypes.NetworkMode(sn.Name()).IsNone() ||
-			containertypes.NetworkMode(n.Name()).IsNone() {
-			return runconfig.ErrConflictNoNetwork
+		if containertypes.NetworkMode(sn.Name()).IsNone() || containertypes.NetworkMode(n.Name()).IsNone() {
+			return cerrdefs.ErrInvalidArgument.WithMessage("container cannot be connected to multiple networks with one of the networks in private (none) mode")
 		}
 	}
 
@@ -216,10 +201,6 @@ func (daemon *Daemon) updateEndpointNetworkSettings(cfg *config.Config, ctr *con
 		return err
 	}
 
-	if ctr.HostConfig.NetworkMode == network.DefaultNetwork {
-		ctr.NetworkSettings.Bridge = cfg.BridgeConfig.Iface
-	}
-
 	return nil
 }
 
@@ -299,11 +280,11 @@ func (daemon *Daemon) findAndAttachNetwork(ctr *container.Container, idOrName st
 
 	var addresses []string
 	if epConfig != nil && epConfig.IPAMConfig != nil {
-		if epConfig.IPAMConfig.IPv4Address != "" {
-			addresses = append(addresses, epConfig.IPAMConfig.IPv4Address)
+		if epConfig.IPAMConfig.IPv4Address.IsValid() {
+			addresses = append(addresses, epConfig.IPAMConfig.IPv4Address.Unmap().String())
 		}
-		if epConfig.IPAMConfig.IPv6Address != "" {
-			addresses = append(addresses, epConfig.IPAMConfig.IPv6Address)
+		if epConfig.IPAMConfig.IPv6Address.IsValid() {
+			addresses = append(addresses, epConfig.IPAMConfig.IPv6Address.Unmap().String())
 		}
 	}
 
@@ -546,43 +527,21 @@ func validateEndpointSettings(nw *libnetwork.Network, nwName string, epConfig *n
 	// TODO(aker): move this into api/types/network/endpoint.go once enableIPOnPredefinedNetwork and
 	//  serviceDiscoveryOnDefaultNetwork are removed.
 	if !containertypes.NetworkMode(nwName).IsUserDefined() {
-		hasStaticAddresses := ipamConfig.IPv4Address != "" || ipamConfig.IPv6Address != ""
+		hasStaticAddresses := ipamConfig.IPv4Address.IsValid() || ipamConfig.IPv6Address.IsValid()
 		// On Linux, user specified IP address is accepted only by networks with user specified subnets.
 		if hasStaticAddresses && !enableIPOnPredefinedNetwork() {
-			errs = append(errs, runconfig.ErrUnsupportedNetworkAndIP)
+			errs = append(errs, cerrdefs.ErrInvalidArgument.WithMessage("user specified IP address is supported on user defined networks only"))
 		}
 		if len(epConfig.Aliases) > 0 && !serviceDiscoveryOnDefaultNetwork() {
-			errs = append(errs, runconfig.ErrUnsupportedNetworkAndAlias)
+			errs = append(errs, cerrdefs.ErrInvalidArgument.WithMessage("network-scoped aliases are only supported for user-defined networks"))
 		}
 	}
 
-	// TODO(aker): add a proper multierror.Append
-	if err := ipamConfig.Validate(); err != nil {
-		errs = append(errs, err.(interface{ Unwrap() []error }).Unwrap()...)
-	}
+	errs = normalizeEndpointIPAMConfig(errs, ipamConfig)
 
 	if nw != nil {
-		_, _, v4Configs, v6Configs := nw.IpamConfig()
-
-		var nwIPv4Subnets, nwIPv6Subnets []networktypes.NetworkSubnet
-		for _, nwIPAMConfig := range v4Configs {
-			nwIPv4Subnets = append(nwIPv4Subnets, nwIPAMConfig)
-		}
-		for _, nwIPAMConfig := range v6Configs {
-			nwIPv6Subnets = append(nwIPv6Subnets, nwIPAMConfig)
-		}
-
-		// TODO(aker): add a proper multierror.Append
-		if err := ipamConfig.IsInRange(nwIPv4Subnets, nwIPv6Subnets); err != nil {
-			errs = append(errs, err.(interface{ Unwrap() []error }).Unwrap()...)
-		}
-	}
-
-	if epConfig.MacAddress != "" {
-		_, err := net.ParseMAC(epConfig.MacAddress)
-		if err != nil {
-			return fmt.Errorf("invalid MAC address %s", epConfig.MacAddress)
-		}
+		v4Info, v6Info := nw.IpamInfo()
+		errs = validateIPAMConfigIsInRange(errs, ipamConfig, v4Info, v6Info)
 	}
 
 	if sysctls, ok := epConfig.DriverOpts[netlabel.EndpointSysctls]; ok {
@@ -607,16 +566,72 @@ func validateEndpointSettings(nw *libnetwork.Network, nwName string, epConfig *n
 	return nil
 }
 
+// normalizeEndpointIPAMConfig checks whether cfg is valid and normalizes cfg in-place.
+func normalizeEndpointIPAMConfig(errs []error, cfg *networktypes.EndpointIPAMConfig) []error {
+	if cfg == nil {
+		return errs
+	}
+
+	if cfg.IPv4Address.IsValid() {
+		if !cfg.IPv4Address.Is4() && !cfg.IPv4Address.Is4In6() || cfg.IPv4Address.IsUnspecified() {
+			errs = append(errs, fmt.Errorf("invalid IPv4 address: %s", cfg.IPv4Address))
+		}
+	}
+	if cfg.IPv6Address.IsValid() {
+		if !cfg.IPv6Address.Is6() || cfg.IPv6Address.Is4In6() || cfg.IPv6Address.IsUnspecified() || cfg.IPv6Address.Zone() != "" {
+			errs = append(errs, fmt.Errorf("invalid IPv6 address: %s", cfg.IPv6Address))
+		}
+	}
+	for _, addr := range cfg.LinkLocalIPs {
+		if !addr.IsValid() || addr.IsUnspecified() {
+			errs = append(errs, fmt.Errorf("invalid link-local IP address: %s", addr))
+		}
+	}
+
+	cfg.IPv4Address = cfg.IPv4Address.Unmap()
+	cfg.IPv6Address = cfg.IPv6Address.Unmap()
+	for i, addr := range cfg.LinkLocalIPs {
+		cfg.LinkLocalIPs[i] = addr.Unmap()
+	}
+
+	return errs
+}
+
+// validateIPAMConfigIsInRange checks whether static IP addresses are valid in a specific network.
+func validateIPAMConfigIsInRange(errs []error, cfg *networktypes.EndpointIPAMConfig, v4Info, v6Info []*libnetwork.IpamInfo) []error {
+	if err := validateEndpointIPAddress(cfg.IPv4Address, v4Info); err != nil {
+		errs = append(errs, err)
+	}
+	if err := validateEndpointIPAddress(cfg.IPv6Address, v6Info); err != nil {
+		errs = append(errs, err)
+	}
+	return errs
+}
+
+func validateEndpointIPAddress(epAddr netip.Addr, ipamInfo []*libnetwork.IpamInfo) error {
+	if !epAddr.IsValid() {
+		return nil
+	}
+
+	for _, subnet := range ipamInfo {
+		if subnet.Pool.Contains(epAddr.AsSlice()) {
+			return nil
+		}
+	}
+
+	return fmt.Errorf("no configured subnet contains IP address %s", epAddr)
+}
+
 // cleanOperationalData resets the operational data from the passed endpoint settings
 func cleanOperationalData(es *network.EndpointSettings) {
 	es.EndpointID = ""
-	es.Gateway = ""
-	es.IPAddress = ""
+	es.Gateway = netip.Addr{}
+	es.IPAddress = netip.Addr{}
 	es.IPPrefixLen = 0
-	es.IPv6Gateway = ""
-	es.GlobalIPv6Address = ""
+	es.IPv6Gateway = netip.Addr{}
+	es.GlobalIPv6Address = netip.Addr{}
 	es.GlobalIPv6PrefixLen = 0
-	es.MacAddress = ""
+	es.MacAddress = nil
 	if es.IPAMOperational {
 		es.IPAMConfig = nil
 	}
@@ -671,7 +686,7 @@ func (daemon *Daemon) connectToNetwork(ctx context.Context, cfg *config.Config,
 	start := time.Now()
 
 	if ctr.HostConfig.NetworkMode.IsContainer() {
-		return runconfig.ErrConflictSharedNetwork
+		return cerrdefs.ErrInvalidArgument.WithMessage("container sharing network namespace with another container or host cannot be connected to any other network")
 	}
 	if cfg.DisableBridge && containertypes.NetworkMode(idOrName).IsBridge() {
 		ctr.Config.NetworkDisabled = true
@@ -701,7 +716,7 @@ func (daemon *Daemon) connectToNetwork(ctx context.Context, cfg *config.Config,
 	endpointConfig.IPAMOperational = false
 	if nwCfg != nil {
 		if epConfig, ok := nwCfg.EndpointsConfig[nwName]; ok {
-			if endpointConfig.IPAMConfig == nil || (endpointConfig.IPAMConfig.IPv4Address == "" && endpointConfig.IPAMConfig.IPv6Address == "" && len(endpointConfig.IPAMConfig.LinkLocalIPs) == 0) {
+			if endpointConfig.IPAMConfig == nil || (!endpointConfig.IPAMConfig.IPv4Address.IsValid() && !endpointConfig.IPAMConfig.IPv6Address.IsValid() && len(endpointConfig.IPAMConfig.LinkLocalIPs) == 0) {
 				endpointConfig.IPAMOperational = true
 			}
 
@@ -720,7 +735,7 @@ func (daemon *Daemon) connectToNetwork(ctx context.Context, cfg *config.Config,
 		return err
 	}
 
-	createOptions, err := buildCreateEndpointOptions(ctr, n, endpointConfig, sb, ipAddresses(cfg.DNS))
+	createOptions, err := buildCreateEndpointOptions(ctr, n, endpointConfig, sb, cfg.DNS)
 	if err != nil {
 		return err
 	}
@@ -736,15 +751,30 @@ func (daemon *Daemon) connectToNetwork(ctx context.Context, cfg *config.Config,
 			}
 		}
 	}()
-	ctr.NetworkSettings.Networks[nwName] = endpointConfig
 
 	delete(ctr.NetworkSettings.Networks, n.ID())
-
-	if err := daemon.updateEndpointNetworkSettings(cfg, ctr, n, ep); err != nil {
-		return err
-	}
+	ctr.NetworkSettings.Networks[nwName] = endpointConfig
+	defer func() {
+		if retErr != nil {
+			delete(ctr.NetworkSettings.Networks, nwName)
+		}
+	}()
 
 	if nwName == network.DefaultNetwork {
+		// Legacy links must be prepared before the Endpoint.Join, because the network
+		// driver needs info about them - and, the daemon's network settings need to be
+		// filled-in for daemon.addLegacyLinks(). So, set up both here.
+		//
+		// However, this means if the Endpoint.Join drops the endpoint's IPv6 address
+		// (because there's a sysctl setting or some equivalent disabling IPv6 on the
+		// interface), host entries set up by addLegacyLinks() for IPv6 addresses of the
+		// linked container will be left behind in the container's /etc/hosts file. It
+		// won't be able to use those addresses, because it won't have IPv6 on that
+		// interface. So, even if the address is recycled by another container on the
+		// network, the old hosts entry can't access the wrong container.
+		if err := daemon.updateEndpointNetworkSettings(cfg, ctr, n, ep); err != nil {
+			return err
+		}
 		if err := daemon.addLegacyLinks(ctx, cfg, ctr, endpointConfig, sb); err != nil {
 			return err
 		}
@@ -755,10 +785,17 @@ func (daemon *Daemon) connectToNetwork(ctx context.Context, cfg *config.Config,
 		return err
 	}
 
+	// Connect the container to the network. Note that this will release the IPv6
+	// address assigned to the Endpoint, if IPv6 is disabled on the interface
+	// (probably by an endpoint specific sysctl setting).
 	if err := ep.Join(ctx, sb, joinOptions...); err != nil {
 		return err
 	}
 
+	if err := daemon.updateEndpointNetworkSettings(cfg, ctr, n, ep); err != nil {
+		return err
+	}
+
 	if !ctr.Managed {
 		// add container name/alias to DNS
 		if err := daemon.ActivateContainerServiceBinding(ctr.Name); err != nil {
@@ -791,11 +828,11 @@ func updateJoinInfo(networkSettings *network.Settings, n *libnetwork.Network, ep
 		// It is not an error to get an empty endpoint info
 		return nil
 	}
-	if epInfo.Gateway() != nil {
-		networkSettings.Networks[n.Name()].Gateway = epInfo.Gateway().String()
+	if gw, ok := netip.AddrFromSlice(epInfo.Gateway()); ok {
+		networkSettings.Networks[n.Name()].Gateway = gw.Unmap()
 	}
-	if epInfo.GatewayIPv6().To16() != nil {
-		networkSettings.Networks[n.Name()].IPv6Gateway = epInfo.GatewayIPv6().String()
+	if gw6, ok := netip.AddrFromSlice(epInfo.GatewayIPv6()); ok && gw6.Is6() {
+		networkSettings.Networks[n.Name()].IPv6Gateway = gw6
 	}
 	return nil
 }
@@ -922,10 +959,10 @@ func (daemon *Daemon) getNetworkedContainer(containerID, connectedContainerPrefi
 		// FIXME (thaJeztah): turns out we don't validate "--network container:<self>" during container create!
 		return nil, errdefs.System(errdefs.InvalidParameter(errors.New("cannot join own network namespace")))
 	}
-	if !nc.IsRunning() {
-		return nil, errdefs.Conflict(fmt.Errorf("cannot join network namespace of a non running container: container %s is %s", strings.TrimPrefix(nc.Name, "/"), nc.StateString()))
+	if !nc.State.IsRunning() {
+		return nil, errdefs.Conflict(fmt.Errorf("cannot join network namespace of a non running container: container %s is %s", strings.TrimPrefix(nc.Name, "/"), nc.State.State()))
 	}
-	if nc.IsRestarting() {
+	if nc.State.IsRestarting() {
 		return nil, fmt.Errorf("cannot join network namespace of container: %w", errContainerIsRestarting(connectedContainerPrefixOrName))
 	}
 	return nc, nil
@@ -998,8 +1035,8 @@ func (daemon *Daemon) ConnectToNetwork(ctx context.Context, ctr *container.Conta
 	ctr.Lock()
 	defer ctr.Unlock()
 
-	if !ctr.Running {
-		if ctr.RemovalInProgress || ctr.Dead {
+	if !ctr.State.Running {
+		if ctr.State.RemovalInProgress || ctr.State.Dead {
 			return errRemovalContainer(ctr.ID)
 		}
 
@@ -1031,8 +1068,8 @@ func (daemon *Daemon) DisconnectFromNetwork(ctx context.Context, ctr *container.
 	ctr.Lock()
 	defer ctr.Unlock()
 
-	if !ctr.Running || (err != nil && force) {
-		if ctr.RemovalInProgress || ctr.Dead {
+	if !ctr.State.Running || (err != nil && force) {
+		if ctr.State.RemovalInProgress || ctr.State.Dead {
 			return errRemovalContainer(ctr.ID)
 		}
 		// In case networkName is resolved we will use n.Name()
@@ -1046,7 +1083,7 @@ func (daemon *Daemon) DisconnectFromNetwork(ctx context.Context, ctr *container.
 		delete(ctr.NetworkSettings.Networks, networkName)
 	} else if err == nil {
 		if ctr.HostConfig.NetworkMode.IsHost() && containertypes.NetworkMode(n.Type()).IsHost() {
-			return runconfig.ErrConflictDisconnectFromHostNetwork
+			return cerrdefs.ErrInvalidArgument.WithMessage("cannot disconnect container from host network - container was created in host network mode")
 		}
 
 		if err := daemon.disconnectFromNetwork(ctx, ctr, n, false); err != nil {
