@@ -5,47 +5,44 @@ import (
 	"fmt"
 	"sync/atomic"
 
-	"github.com/containerd/containerd"
-	"github.com/containerd/containerd/content"
-	cerrdefs "github.com/containerd/containerd/errdefs"
-	"github.com/containerd/containerd/images"
-	"github.com/containerd/containerd/plugin"
-	"github.com/containerd/containerd/remotes/docker"
-	"github.com/containerd/containerd/snapshots"
+	containerd "github.com/containerd/containerd/v2/client"
+	"github.com/containerd/containerd/v2/core/content"
+	c8dimages "github.com/containerd/containerd/v2/core/images"
+	"github.com/containerd/containerd/v2/core/remotes/docker"
+	"github.com/containerd/containerd/v2/core/snapshots"
+	"github.com/containerd/containerd/v2/plugins"
+	cerrdefs "github.com/containerd/errdefs"
 	"github.com/containerd/log"
-	"github.com/distribution/reference"
-	"github.com/docker/docker/container"
-	daemonevents "github.com/docker/docker/daemon/events"
-	dimages "github.com/docker/docker/daemon/images"
-	"github.com/docker/docker/daemon/snapshotter"
-	"github.com/docker/docker/errdefs"
-	"github.com/docker/docker/layer"
-	"github.com/docker/docker/pkg/idtools"
-	"github.com/docker/docker/registry"
+	"github.com/containerd/platforms"
+	"github.com/moby/moby/v2/daemon/container"
+	daemonevents "github.com/moby/moby/v2/daemon/events"
+	dimages "github.com/moby/moby/v2/daemon/images"
+	"github.com/moby/moby/v2/daemon/internal/distribution"
+	"github.com/moby/moby/v2/daemon/snapshotter"
+	"github.com/moby/moby/v2/errdefs"
+	"github.com/moby/sys/user"
+	"github.com/opencontainers/go-digest"
+	ocispec "github.com/opencontainers/image-spec/specs-go/v1"
 	"github.com/pkg/errors"
 )
 
 // ImageService implements daemon.ImageService
 type ImageService struct {
 	client              *containerd.Client
-	images              images.Store
+	images              c8dimages.Store
 	content             content.Store
 	containers          container.Store
 	snapshotterServices map[string]snapshots.Snapshotter
 	snapshotter         string
 	registryHosts       docker.RegistryHosts
-	registryService     registryResolver
+	registryService     distribution.RegistryResolver
 	eventsService       *daemonevents.Events
 	pruneRunning        atomic.Bool
 	refCountMounter     snapshotter.Mounter
-	idMapping           idtools.IdentityMapping
-}
+	idMapping           user.IdentityMapping
 
-type registryResolver interface {
-	IsInsecureRegistry(host string) bool
-	ResolveRepository(name reference.Named) (*registry.RepositoryInfo, error)
-	LookupPullEndpoints(hostname string) ([]registry.APIEndpoint, error)
-	LookupPushEndpoints(hostname string) ([]registry.APIEndpoint, error)
+	// defaultPlatformOverride is used in tests to override the host platform.
+	defaultPlatformOverride platforms.MatchComparer
 }
 
 type ImageServiceConfig struct {
@@ -53,10 +50,10 @@ type ImageServiceConfig struct {
 	Containers      container.Store
 	Snapshotter     string
 	RegistryHosts   docker.RegistryHosts
-	Registry        registryResolver
+	Registry        distribution.RegistryResolver
 	EventsService   *daemonevents.Events
 	RefCountMounter snapshotter.Mounter
-	IDMapping       idtools.IdentityMapping
+	IDMapping       user.IdentityMapping
 }
 
 // NewService creates a new ImageService.
@@ -101,14 +98,15 @@ func (i *ImageService) CountImages(ctx context.Context) int {
 		return 0
 	}
 
-	return len(imgs)
-}
+	uniqueImages := map[digest.Digest]struct{}{}
+	for _, i := range imgs {
+		dgst := i.Target().Digest
+		if _, ok := uniqueImages[dgst]; !ok {
+			uniqueImages[dgst] = struct{}{}
+		}
+	}
 
-// CreateLayer creates a filesystem layer for a container.
-// called from create.go
-// TODO: accept an opt struct instead of container?
-func (i *ImageService) CreateLayer(container *container.Container, initFunc layer.MountInit) (layer.RWLayer, error) {
-	return nil, errdefs.NotImplemented(errdefs.NotImplemented(errors.New("not implemented")))
+	return len(uniqueImages)
 }
 
 // LayerStoreStatus returns the status for each layer store
@@ -116,12 +114,12 @@ func (i *ImageService) CreateLayer(container *container.Container, initFunc laye
 func (i *ImageService) LayerStoreStatus() [][2]string {
 	// TODO(thaJeztah) do we want to add more details about the driver here?
 	return [][2]string{
-		{"driver-type", string(plugin.SnapshotPlugin)},
+		{"driver-type", string(plugins.SnapshotPlugin)},
 	}
 }
 
 // GetLayerMountID returns the mount ID for a layer
-// called from daemon.go Daemon.Shutdown(), and Daemon.Cleanup() (cleanup is actually continerCleanup)
+// called from daemon.go Daemon.Shutdown(), and Daemon.Cleanup() (cleanup is actually containerCleanup)
 // TODO: needs to be refactored to Unmount (see callers), or removed and replaced with GetLayerByID
 func (i *ImageService) GetLayerMountID(cid string) (string, error) {
 	return "", errdefs.NotImplemented(errors.New("not implemented"))
@@ -139,19 +137,43 @@ func (i *ImageService) StorageDriver() string {
 	return i.snapshotter
 }
 
-// ReleaseLayer releases a layer allowing it to be removed
-// called from delete.go Daemon.cleanupContainer(), and Daemon.containerExport()
-func (i *ImageService) ReleaseLayer(rwlayer layer.RWLayer) error {
-	return errdefs.NotImplemented(errors.New("not implemented"))
+// ImageDiskUsage returns the number of bytes used by content and layer stores
+// called from disk_usage.go
+func (i *ImageService) ImageDiskUsage(ctx context.Context) (int64, error) {
+	diskUsage, err := i.layerDiskUsage(ctx)
+	if err != nil {
+		return 0, err
+	}
+
+	// Include the size of content size from the images.
+	imgs, err := i.images.List(ctx)
+	if err != nil {
+		return 0, err
+	}
+
+	visitedImages := make(map[digest.Digest]struct{})
+	for _, img := range imgs {
+		if err := i.walkPresentChildren(ctx, img.Target, func(ctx context.Context, desc ocispec.Descriptor) error {
+			if _, ok := visitedImages[desc.Digest]; ok {
+				return nil
+			}
+			visitedImages[desc.Digest] = struct{}{}
+
+			diskUsage += desc.Size
+			return nil
+		}); err != nil {
+			return 0, err
+		}
+	}
+	return diskUsage, nil
 }
 
 // LayerDiskUsage returns the number of bytes used by layer stores
 // called from disk_usage.go
-func (i *ImageService) LayerDiskUsage(ctx context.Context) (int64, error) {
-	var allLayersSize int64
+func (i *ImageService) layerDiskUsage(ctx context.Context) (allLayersSize int64, err error) {
 	// TODO(thaJeztah): do we need to take multiple snapshotters into account? See https://github.com/moby/moby/issues/45273
 	snapshotter := i.client.SnapshotService(i.snapshotter)
-	snapshotter.Walk(ctx, func(ctx context.Context, info snapshots.Info) error {
+	err = snapshotter.Walk(ctx, func(ctx context.Context, info snapshots.Info) error {
 		usage, err := snapshotter.Usage(ctx, info.Name)
 		if err != nil {
 			return err
@@ -159,7 +181,7 @@ func (i *ImageService) LayerDiskUsage(ctx context.Context) (int64, error) {
 		allLayersSize += usage.Size
 		return nil
 	})
-	return allLayersSize, nil
+	return allLayersSize, err
 }
 
 // UpdateConfig values
