@@ -1,4 +1,4 @@
-package daemon // import "github.com/docker/docker/daemon"
+package daemon
 
 import (
 	"context"
@@ -8,16 +8,14 @@ import (
 	"strings"
 	"time"
 
-	cerrdefs "github.com/containerd/containerd/errdefs"
-	"github.com/containerd/containerd/leases"
 	"github.com/containerd/log"
-	"github.com/docker/docker/api/types/backend"
-	containertypes "github.com/docker/docker/api/types/container"
-	"github.com/docker/docker/api/types/events"
-	"github.com/docker/docker/container"
-	"github.com/docker/docker/daemon/config"
-	"github.com/docker/docker/errdefs"
-	"github.com/docker/docker/pkg/containerfs"
+	"github.com/moby/moby/api/types/events"
+	"github.com/moby/moby/v2/daemon/config"
+	"github.com/moby/moby/v2/daemon/container"
+	"github.com/moby/moby/v2/daemon/internal/containerfs"
+	"github.com/moby/moby/v2/daemon/internal/metrics"
+	"github.com/moby/moby/v2/daemon/server/backend"
+	"github.com/moby/moby/v2/errdefs"
 	"github.com/opencontainers/selinux/go-selinux"
 	"github.com/pkg/errors"
 )
@@ -38,11 +36,11 @@ func (daemon *Daemon) containerRm(cfg *config.Config, name string, opts *backend
 	}
 
 	// Container state RemovalInProgress should be used to avoid races.
-	if inProgress := ctr.SetRemovalInProgress(); inProgress {
+	if inProgress := ctr.State.SetRemovalInProgress(); inProgress {
 		err := fmt.Errorf("removal of container %s is already in progress", name)
 		return errdefs.Conflict(err)
 	}
-	defer ctr.ResetRemovalInProgress()
+	defer ctr.State.ResetRemovalInProgress()
 
 	// check if container wasn't deregistered by previous rm since Get
 	if c := daemon.containers.Get(ctr.ID); c == nil {
@@ -54,30 +52,31 @@ func (daemon *Daemon) containerRm(cfg *config.Config, name string, opts *backend
 	}
 
 	err = daemon.cleanupContainer(ctr, *opts)
-	containerActions.WithValues("delete").UpdateSince(start)
-
-	return err
+	metrics.ContainerActions.WithValues("delete").UpdateSince(start)
+	if err != nil {
+		return fmt.Errorf("cannot remove container %q: %w", name, err)
+	}
+	return nil
 }
 
-func (daemon *Daemon) rmLink(cfg *config.Config, container *container.Container, name string) error {
+func (daemon *Daemon) rmLink(cfg *config.Config, ctr *container.Container, name string) error {
 	if name[0] != '/' {
 		name = "/" + name
 	}
 	parent, n := path.Split(name)
 	if parent == "/" {
-		return fmt.Errorf("Conflict, cannot remove the default link name of the container")
+		return errors.New("Conflict, cannot remove the default link name of the container")
 	}
 
 	parent = strings.TrimSuffix(parent, "/")
-	pe, err := daemon.containersReplica.Snapshot().GetID(parent)
+	parentID, err := daemon.containersReplica.Snapshot().GetID(parent)
 	if err != nil {
 		return fmt.Errorf("Cannot get parent %s for link name %s", parent, name)
 	}
 
 	daemon.releaseName(name)
-	parentContainer, _ := daemon.GetContainer(pe)
-	if parentContainer != nil {
-		daemon.linkIndex.unlink(name, container, parentContainer)
+	if parentContainer := daemon.containers.Get(parentID); parentContainer != nil {
+		daemon.linkIndex.unlink(name, ctr, parentContainer)
 		if err := daemon.updateNetwork(cfg, parentContainer); err != nil {
 			log.G(context.TODO()).Debugf("Could not update network to remove link %s: %v", n, err)
 		}
@@ -87,23 +86,23 @@ func (daemon *Daemon) rmLink(cfg *config.Config, container *container.Container,
 
 // cleanupContainer unregisters a container from the daemon, stops stats
 // collection and cleanly removes contents and metadata from the filesystem.
-func (daemon *Daemon) cleanupContainer(container *container.Container, config backend.ContainerRmConfig) error {
-	if container.IsRunning() {
+func (daemon *Daemon) cleanupContainer(ctr *container.Container, config backend.ContainerRmConfig) error {
+	if ctr.State.IsRunning() {
 		if !config.ForceRemove {
-			if state := container.StateString(); state == "paused" {
-				return errdefs.Conflict(fmt.Errorf("cannot remove container %q: container is %s and must be unpaused first", container.Name, state))
+			if ctr.State.Paused {
+				return errdefs.Conflict(errors.New("container is paused and must be unpaused first"))
 			} else {
-				return errdefs.Conflict(fmt.Errorf("cannot remove container %q: container is %s: stop the container before removing or force remove", container.Name, state))
+				return errdefs.Conflict(fmt.Errorf("container is %s: stop the container before removing or force remove", ctr.State.State()))
 			}
 		}
-		if err := daemon.Kill(container); err != nil && !isNotRunning(err) {
-			return fmt.Errorf("cannot remove container %q: could not kill: %w", container.Name, err)
+		if err := daemon.Kill(ctr); err != nil && !isNotRunning(err) {
+			return fmt.Errorf("could not kill container: %w", err)
 		}
 	}
 
 	// stop collection of stats for the container regardless
 	// if stats are currently getting collected.
-	daemon.statsCollector.StopCollection(container)
+	daemon.statsCollector.StopCollection(ctr)
 
 	// stopTimeout is the number of seconds to wait for the container to stop
 	// gracefully before forcibly killing it.
@@ -117,72 +116,67 @@ func (daemon *Daemon) cleanupContainer(container *container.Container, config ba
 	// If you arrived here and know the answer, you earned yourself a picture
 	// of a cute animal of your own choosing.
 	stopTimeout := 3
-	if err := daemon.containerStop(context.TODO(), container, containertypes.StopOptions{Timeout: &stopTimeout}); err != nil {
+	if err := daemon.containerStop(context.TODO(), ctr, backend.ContainerStopOptions{Timeout: &stopTimeout}); err != nil {
 		return err
 	}
 
 	// Mark container dead. We don't want anybody to be restarting it.
-	container.Lock()
-	container.Dead = true
+	ctr.Lock()
+	ctr.State.Dead = true
+
+	// Copy RWLayer for releasing and clear the reference while holding the container lock.
+	rwLayer := ctr.RWLayer
+	ctr.RWLayer = nil
 
 	// Save container state to disk. So that if error happens before
 	// container meta file got removed from disk, then a restart of
 	// docker should not make a dead container alive.
-	if err := container.CheckpointTo(daemon.containersReplica); err != nil && !os.IsNotExist(err) {
+	if err := ctr.CheckpointTo(context.WithoutCancel(context.TODO()), daemon.containersReplica); err != nil && !os.IsNotExist(err) {
 		log.G(context.TODO()).Errorf("Error saving dying container to disk: %v", err)
 	}
-	container.Unlock()
+	ctr.Unlock()
 
 	// When container creation fails and `RWLayer` has not been created yet, we
 	// do not call `ReleaseRWLayer`
-	if container.RWLayer != nil {
-		if err := daemon.imageService.ReleaseLayer(container.RWLayer); err != nil {
-			err = errors.Wrapf(err, "container %s", container.ID)
-			container.SetRemovalError(err)
+	if rwLayer != nil {
+		if err := daemon.imageService.ReleaseLayer(rwLayer); err != nil {
+			// Restore the reference on error as it possibly was not released.
+			ctr.Lock()
+			ctr.RWLayer = rwLayer
+			ctr.Unlock()
+			ctr.State.SetRemovalError(err)
 			return err
 		}
-		container.RWLayer = nil
-	} else {
-		if daemon.UsesSnapshotter() {
-			ls := daemon.containerdClient.LeasesService()
-			lease := leases.Lease{
-				ID: container.ID,
-			}
-			if err := ls.Delete(context.Background(), lease, leases.SynchronousDelete); err != nil {
-				if !cerrdefs.IsNotFound(err) {
-					container.SetRemovalError(err)
-					return err
-				}
-			}
-		}
 	}
 
 	// Hold the container lock while deleting the container root directory
 	// so that other goroutines don't attempt to concurrently open files
 	// within it. Having any file open on Windows (without the
 	// FILE_SHARE_DELETE flag) will block it from being deleted.
-	container.Lock()
-	err := containerfs.EnsureRemoveAll(container.Root)
-	container.Unlock()
+	//
+	// TODO(thaJeztah): should this be moved to the "container" itself, or possibly be delegated to the graphdriver or snapshotter?
+	ctr.Lock()
+	err := containerfs.EnsureRemoveAll(ctr.Root)
+	ctr.Unlock()
 	if err != nil {
-		err = errors.Wrapf(err, "unable to remove filesystem for %s", container.ID)
-		container.SetRemovalError(err)
+		err = errors.Wrap(err, "unable to remove filesystem")
+		ctr.State.SetRemovalError(err)
 		return err
 	}
 
-	linkNames := daemon.linkIndex.delete(container)
-	selinux.ReleaseLabel(container.ProcessLabel)
-	daemon.containers.Delete(container.ID)
-	daemon.containersReplica.Delete(container)
-	if err := daemon.removeMountPoints(container, config.RemoveVolume); err != nil {
+	linkNames := daemon.linkIndex.delete(ctr)
+	selinux.ReleaseLabel(ctr.ProcessLabel)
+	daemon.containers.Delete(ctr.ID)
+	daemon.containersReplica.Delete(ctr)
+	if err := daemon.removeMountPoints(ctr, config.RemoveVolume); err != nil {
 		log.G(context.TODO()).Error(err)
 	}
 	for _, name := range linkNames {
 		daemon.releaseName(name)
 	}
-	container.SetRemoved()
-	stateCtr.del(container.ID)
+	ctr.State.SetRemoved()
+	metrics.StateCtr.Delete(ctr.ID)
 
-	daemon.LogContainerEvent(container, events.ActionDestroy)
+	daemon.LogContainerEvent(ctr, events.ActionDestroy)
 	return nil
 }
