@@ -14,16 +14,16 @@ import (
 	"testing"
 	"time"
 
-	"github.com/docker/docker/integration-cli/cli"
-	"github.com/docker/docker/integration-cli/daemon"
-	"github.com/docker/docker/integration-cli/environment"
-	"github.com/docker/docker/internal/test/suite"
-	"github.com/docker/docker/testutil"
-	testdaemon "github.com/docker/docker/testutil/daemon"
-	ienv "github.com/docker/docker/testutil/environment"
-	"github.com/docker/docker/testutil/fakestorage"
-	"github.com/docker/docker/testutil/fixtures/plugin"
-	"github.com/docker/docker/testutil/registry"
+	"github.com/moby/moby/v2/integration-cli/cli"
+	"github.com/moby/moby/v2/integration-cli/daemon"
+	"github.com/moby/moby/v2/integration-cli/environment"
+	"github.com/moby/moby/v2/internal/test/suite"
+	"github.com/moby/moby/v2/internal/testutil"
+	testdaemon "github.com/moby/moby/v2/internal/testutil/daemon"
+	ienv "github.com/moby/moby/v2/internal/testutil/environment"
+	"github.com/moby/moby/v2/internal/testutil/fakestorage"
+	"github.com/moby/moby/v2/internal/testutil/fixtures/plugin"
+	"github.com/moby/moby/v2/internal/testutil/registry"
 	"go.opentelemetry.io/otel"
 	"go.opentelemetry.io/otel/attribute"
 	"go.opentelemetry.io/otel/codes"
@@ -418,29 +418,28 @@ func TestDockerHubPullSuite(t *testing.T) {
 
 type DockerSuite struct{}
 
-func (s *DockerSuite) OnTimeout(c *testing.T) {
+func (s *DockerSuite) OnTimeout(t *testing.T) {
 	if testEnv.IsRemoteDaemon() {
 		return
 	}
-	path := filepath.Join(os.Getenv("DEST"), "docker.pid")
-	b, err := os.ReadFile(path)
+	pidFile := filepath.Join(os.Getenv("DEST"), "docker.pid")
+	b, err := os.ReadFile(pidFile)
 	if err != nil {
-		c.Fatalf("Failed to get daemon PID from %s\n", path)
+		t.Fatalf("Failed to get daemon PID from %s\n", pidFile)
 	}
 
 	rawPid, err := strconv.ParseInt(string(b), 10, 32)
 	if err != nil {
-		c.Fatalf("Failed to parse pid from %s: %s\n", path, err)
+		t.Fatalf("Failed to parse pid from %s: %s\n", pidFile, err)
 	}
 
-	daemonPid := int(rawPid)
-	if daemonPid > 0 {
+	if daemonPid := int(rawPid); daemonPid > 0 {
 		testdaemon.SignalDaemonDump(daemonPid)
 	}
 }
 
-func (s *DockerSuite) TearDownTest(ctx context.Context, c *testing.T) {
-	testEnv.Clean(ctx, c)
+func (s *DockerSuite) TearDownTest(ctx context.Context, t *testing.T) {
+	testEnv.Clean(ctx, t)
 }
 
 type DockerRegistrySuite struct {
@@ -449,25 +448,25 @@ type DockerRegistrySuite struct {
 	d   *daemon.Daemon
 }
 
-func (s *DockerRegistrySuite) OnTimeout(c *testing.T) {
+func (s *DockerRegistrySuite) OnTimeout(t *testing.T) {
 	s.d.DumpStackAndQuit()
 }
 
-func (s *DockerRegistrySuite) SetUpTest(ctx context.Context, c *testing.T) {
-	testRequires(c, DaemonIsLinux, RegistryHosting, testEnv.IsLocalDaemon)
-	s.reg = registry.NewV2(c)
-	s.reg.WaitReady(c)
-	s.d = daemon.New(c, dockerBinary, dockerdBinary, testdaemon.WithEnvironment(testEnv.Execution))
+func (s *DockerRegistrySuite) SetUpTest(ctx context.Context, t *testing.T) {
+	testRequires(t, DaemonIsLinux, RegistryHosting, testEnv.IsLocalDaemon)
+	s.reg = registry.NewV2(t)
+	s.reg.WaitReady(t)
+	s.d = daemon.New(t, dockerBinary, dockerdBinary, testdaemon.WithEnvironment(testEnv.Execution))
 }
 
-func (s *DockerRegistrySuite) TearDownTest(ctx context.Context, c *testing.T) {
+func (s *DockerRegistrySuite) TearDownTest(ctx context.Context, t *testing.T) {
 	if s.reg != nil {
 		s.reg.Close()
 	}
 	if s.d != nil {
-		s.d.Stop(c)
+		s.d.Stop(t)
 	}
-	s.ds.TearDownTest(ctx, c)
+	s.ds.TearDownTest(ctx, t)
 }
 
 type DockerRegistryAuthHtpasswdSuite struct {
@@ -476,27 +475,27 @@ type DockerRegistryAuthHtpasswdSuite struct {
 	d   *daemon.Daemon
 }
 
-func (s *DockerRegistryAuthHtpasswdSuite) OnTimeout(c *testing.T) {
+func (s *DockerRegistryAuthHtpasswdSuite) OnTimeout(t *testing.T) {
 	s.d.DumpStackAndQuit()
 }
 
-func (s *DockerRegistryAuthHtpasswdSuite) SetUpTest(ctx context.Context, c *testing.T) {
-	testRequires(c, DaemonIsLinux, RegistryHosting, testEnv.IsLocalDaemon)
-	s.reg = registry.NewV2(c, registry.Htpasswd)
-	s.reg.WaitReady(c)
-	s.d = daemon.New(c, dockerBinary, dockerdBinary, testdaemon.WithEnvironment(testEnv.Execution))
+func (s *DockerRegistryAuthHtpasswdSuite) SetUpTest(ctx context.Context, t *testing.T) {
+	testRequires(t, DaemonIsLinux, RegistryHosting, testEnv.IsLocalDaemon)
+	s.reg = registry.NewV2(t, registry.Htpasswd)
+	s.reg.WaitReady(t)
+	s.d = daemon.New(t, dockerBinary, dockerdBinary, testdaemon.WithEnvironment(testEnv.Execution))
 }
 
-func (s *DockerRegistryAuthHtpasswdSuite) TearDownTest(ctx context.Context, c *testing.T) {
+func (s *DockerRegistryAuthHtpasswdSuite) TearDownTest(ctx context.Context, t *testing.T) {
 	if s.reg != nil {
 		out, err := s.d.Cmd("logout", privateRegistryURL)
-		assert.NilError(c, err, out)
+		assert.NilError(t, err, out)
 		s.reg.Close()
 	}
 	if s.d != nil {
-		s.d.Stop(c)
+		s.d.Stop(t)
 	}
-	s.ds.TearDownTest(ctx, c)
+	s.ds.TearDownTest(ctx, t)
 }
 
 type DockerRegistryAuthTokenSuite struct {
@@ -505,33 +504,33 @@ type DockerRegistryAuthTokenSuite struct {
 	d   *daemon.Daemon
 }
 
-func (s *DockerRegistryAuthTokenSuite) OnTimeout(c *testing.T) {
+func (s *DockerRegistryAuthTokenSuite) OnTimeout(t *testing.T) {
 	s.d.DumpStackAndQuit()
 }
 
-func (s *DockerRegistryAuthTokenSuite) SetUpTest(ctx context.Context, c *testing.T) {
-	testRequires(c, DaemonIsLinux, RegistryHosting, testEnv.IsLocalDaemon)
-	s.d = daemon.New(c, dockerBinary, dockerdBinary, testdaemon.WithEnvironment(testEnv.Execution))
+func (s *DockerRegistryAuthTokenSuite) SetUpTest(ctx context.Context, t *testing.T) {
+	testRequires(t, DaemonIsLinux, RegistryHosting, testEnv.IsLocalDaemon)
+	s.d = daemon.New(t, dockerBinary, dockerdBinary, testdaemon.WithEnvironment(testEnv.Execution))
 }
 
-func (s *DockerRegistryAuthTokenSuite) TearDownTest(ctx context.Context, c *testing.T) {
+func (s *DockerRegistryAuthTokenSuite) TearDownTest(ctx context.Context, t *testing.T) {
 	if s.reg != nil {
 		out, err := s.d.Cmd("logout", privateRegistryURL)
-		assert.NilError(c, err, out)
+		assert.NilError(t, err, out)
 		s.reg.Close()
 	}
 	if s.d != nil {
-		s.d.Stop(c)
+		s.d.Stop(t)
 	}
-	s.ds.TearDownTest(ctx, c)
+	s.ds.TearDownTest(ctx, t)
 }
 
-func (s *DockerRegistryAuthTokenSuite) setupRegistryWithTokenService(c *testing.T, tokenURL string) {
+func (s *DockerRegistryAuthTokenSuite) setupRegistryWithTokenService(t *testing.T, tokenURL string) {
 	if s == nil {
-		c.Fatal("registry suite isn't initialized")
+		t.Fatal("registry suite isn't initialized")
 	}
-	s.reg = registry.NewV2(c, registry.Token(tokenURL))
-	s.reg.WaitReady(c)
+	s.reg = registry.NewV2(t, registry.Token(tokenURL))
+	s.reg.WaitReady(t)
 }
 
 type DockerDaemonSuite struct {
@@ -539,24 +538,24 @@ type DockerDaemonSuite struct {
 	d  *daemon.Daemon
 }
 
-func (s *DockerDaemonSuite) OnTimeout(c *testing.T) {
+func (s *DockerDaemonSuite) OnTimeout(t *testing.T) {
 	s.d.DumpStackAndQuit()
 }
 
-func (s *DockerDaemonSuite) SetUpTest(ctx context.Context, c *testing.T) {
-	testRequires(c, DaemonIsLinux, testEnv.IsLocalDaemon)
-	s.d = daemon.New(c, dockerBinary, dockerdBinary, testdaemon.WithEnvironment(testEnv.Execution))
+func (s *DockerDaemonSuite) SetUpTest(ctx context.Context, t *testing.T) {
+	testRequires(t, DaemonIsLinux, testEnv.IsLocalDaemon)
+	s.d = daemon.New(t, dockerBinary, dockerdBinary, testdaemon.WithEnvironment(testEnv.Execution))
 }
 
-func (s *DockerDaemonSuite) TearDownTest(ctx context.Context, c *testing.T) {
-	testRequires(c, DaemonIsLinux, testEnv.IsLocalDaemon)
+func (s *DockerDaemonSuite) TearDownTest(ctx context.Context, t *testing.T) {
+	testRequires(t, DaemonIsLinux, testEnv.IsLocalDaemon)
 	if s.d != nil {
-		s.d.Stop(c)
+		s.d.Stop(t)
 	}
-	s.ds.TearDownTest(ctx, c)
+	s.ds.TearDownTest(ctx, t)
 }
 
-func (s *DockerDaemonSuite) TearDownSuite(ctx context.Context, c *testing.T) {
+func (s *DockerDaemonSuite) TearDownSuite(ctx context.Context, t *testing.T) {
 	filepath.Walk(testdaemon.SockRoot, func(path string, fi os.FileInfo, err error) error {
 		if err != nil {
 			// ignore errors here
@@ -581,7 +580,7 @@ type DockerSwarmSuite struct {
 	portIndex   int
 }
 
-func (s *DockerSwarmSuite) OnTimeout(c *testing.T) {
+func (s *DockerSwarmSuite) OnTimeout(t *testing.T) {
 	s.daemonsLock.Lock()
 	defer s.daemonsLock.Unlock()
 	for _, d := range s.daemons {
@@ -589,24 +588,24 @@ func (s *DockerSwarmSuite) OnTimeout(c *testing.T) {
 	}
 }
 
-func (s *DockerSwarmSuite) SetUpTest(ctx context.Context, c *testing.T) {
-	testRequires(c, DaemonIsLinux, testEnv.IsLocalDaemon)
+func (s *DockerSwarmSuite) SetUpTest(ctx context.Context, t *testing.T) {
+	testRequires(t, DaemonIsLinux, testEnv.IsLocalDaemon)
 }
 
-func (s *DockerSwarmSuite) AddDaemon(ctx context.Context, c *testing.T, joinSwarm, manager bool) *daemon.Daemon {
-	c.Helper()
-	d := daemon.New(c, dockerBinary, dockerdBinary,
+func (s *DockerSwarmSuite) AddDaemon(ctx context.Context, t *testing.T, joinSwarm, manager bool) *daemon.Daemon {
+	t.Helper()
+	d := daemon.New(t, dockerBinary, dockerdBinary,
 		testdaemon.WithEnvironment(testEnv.Execution),
 		testdaemon.WithSwarmPort(defaultSwarmPort+s.portIndex),
 	)
 	if joinSwarm {
 		if len(s.daemons) > 0 {
-			d.StartAndSwarmJoin(ctx, c, s.daemons[0].Daemon, manager)
+			d.StartAndSwarmJoin(ctx, t, s.daemons[0].Daemon, manager)
 		} else {
-			d.StartAndSwarmInit(ctx, c)
+			d.StartAndSwarmInit(ctx, t)
 		}
 	} else {
-		d.StartNodeWithBusybox(ctx, c)
+		d.StartNodeWithBusybox(ctx, t)
 	}
 
 	s.daemonsLock.Lock()
@@ -617,22 +616,22 @@ func (s *DockerSwarmSuite) AddDaemon(ctx context.Context, c *testing.T, joinSwar
 	return d
 }
 
-func (s *DockerSwarmSuite) TearDownTest(ctx context.Context, c *testing.T) {
-	testRequires(c, DaemonIsLinux)
+func (s *DockerSwarmSuite) TearDownTest(ctx context.Context, t *testing.T) {
+	testRequires(t, DaemonIsLinux)
 	s.daemonsLock.Lock()
 	for _, d := range s.daemons {
 		if d != nil {
-			if c.Failed() {
-				d.TailLogsT(c, 100)
+			if t.Failed() {
+				d.TailLogsT(t, 100)
 			}
-			d.Stop(c)
-			d.Cleanup(c)
+			d.Stop(t)
+			d.Cleanup(t)
 		}
 	}
 	s.daemons = nil
 	s.portIndex = 0
 	s.daemonsLock.Unlock()
-	s.ds.TearDownTest(ctx, c)
+	s.ds.TearDownTest(ctx, t)
 }
 
 type DockerPluginSuite struct {
@@ -652,28 +651,28 @@ func (ps *DockerPluginSuite) getPluginRepoWithTag() string {
 	return ps.getPluginRepo() + ":" + "latest"
 }
 
-func (ps *DockerPluginSuite) SetUpSuite(ctx context.Context, c *testing.T) {
-	testRequires(c, DaemonIsLinux, RegistryHosting)
-	ps.registry = registry.NewV2(c)
-	ps.registry.WaitReady(c)
+func (ps *DockerPluginSuite) SetUpSuite(ctx context.Context, t *testing.T) {
+	testRequires(t, DaemonIsLinux, RegistryHosting)
+	ps.registry = registry.NewV2(t)
+	ps.registry.WaitReady(t)
 
 	ctx, cancel := context.WithTimeout(ctx, 60*time.Second)
 	defer cancel()
 
 	err := plugin.CreateInRegistry(ctx, ps.getPluginRepo(), nil)
-	assert.NilError(c, err, "failed to create plugin")
+	assert.NilError(t, err, "failed to create plugin")
 }
 
-func (ps *DockerPluginSuite) TearDownSuite(ctx context.Context, c *testing.T) {
+func (ps *DockerPluginSuite) TearDownSuite(ctx context.Context, t *testing.T) {
 	if ps.registry != nil {
 		ps.registry.Close()
 	}
 }
 
-func (ps *DockerPluginSuite) TearDownTest(ctx context.Context, c *testing.T) {
-	ps.ds.TearDownTest(ctx, c)
+func (ps *DockerPluginSuite) TearDownTest(ctx context.Context, t *testing.T) {
+	ps.ds.TearDownTest(ctx, t)
 }
 
-func (ps *DockerPluginSuite) OnTimeout(c *testing.T) {
-	ps.ds.OnTimeout(c)
+func (ps *DockerPluginSuite) OnTimeout(t *testing.T) {
+	ps.ds.OnTimeout(t)
 }
