@@ -5,9 +5,9 @@ package logging
 import (
 	"testing"
 
-	"github.com/docker/docker/api/types"
-	"github.com/docker/docker/testutil"
-	"github.com/docker/docker/testutil/daemon"
+	"github.com/moby/moby/client"
+	"github.com/moby/moby/v2/internal/testutil"
+	"github.com/moby/moby/v2/internal/testutil/daemon"
 	"gotest.tools/v3/assert"
 	"gotest.tools/v3/skip"
 )
@@ -23,16 +23,18 @@ func TestDaemonStartWithLogOpt(t *testing.T) {
 	ctx := testutil.StartSpan(baseContext, t)
 
 	d := daemon.New(t)
-	d.Start(t, "--iptables=false")
+	d.Start(t, "--iptables=false", "--ip6tables=false")
 	defer d.Stop(t)
 
 	c := d.NewClientT(t)
 
 	createPlugin(ctx, t, c, "test", "dummy", asLogDriver)
-	err := c.PluginEnable(ctx, "test", types.PluginEnableOptions{Timeout: 30})
+	_, err := c.PluginEnable(ctx, "test", client.PluginEnableOptions{Timeout: 30})
 	assert.Check(t, err)
-	defer c.PluginRemove(ctx, "test", types.PluginRemoveOptions{Force: true})
+	defer func() {
+		_, _ = c.PluginRemove(ctx, "test", client.PluginRemoveOptions{Force: true})
+	}()
 
 	d.Stop(t)
-	d.Start(t, "--iptables=false", "--log-driver=test", "--log-opt=foo=bar")
+	d.Start(t, "--iptables=false", "--ip6tables=false", "--log-driver=test", "--log-opt=foo=bar")
 }
